/*
Just to visualize how the swarm "moves" in a carefully chosen 2D space
You need the free Leonardo compiler
http://www.dis.uniroma1.it/~demetres/Leonardo/
just for Mac, for the moment (2001-08)

Data file.
For each iteration
N (swarm size) 
	For each particle
	X1 X2 F (two "coordinates" and a function value) X1 X2 F (the same, but for the best previous position



*/

#include <math.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#ifdef LEONARDO
	#include <syscall.h>
#endif

#define Max_DD	76
#define	Max_Iter 200
#define Max_swarm_size 	101

// Structures
struct	frame		{float s[5];float scale1;float scale2;float min1;float min2;float max2;float fmin;};
struct	graph		{int size; float M[Max_DD][Max_DD];};
struct	visu_part	{int LABEL;float X1;float X2;float F;float XP1;float XP2;float FP;int LABEL_H;};	

// Global variables
float				COS[Max_DD];
int					DIM;
struct frame		frame={0};
struct graph		G;
float				L[Max_Iter][2]; // Trajectory of the best particle
float				LP[Max_Iter][2]; 
int					LABEL_BEST;
float				NA=-999999; // Big negative value
int					NB_BEST=0;
struct visu_part	PART[Max_swarm_size]={0};
float				PI=3.1415926535;
float				POS_BEST[Max_DD];
float				SCALE=200; // Side of the square where particles are visualized
float				SIN[Max_DD];
int					SWSIZE;
float				XMAX;
float				XMIN;

/**
     // Declarations
     PointLabel(ID,Out T,Out S,V) Standard _User;
     PointLabelColor(ID,Out C,V)  Standard _User;
     PointLabelFace(ID,Out F,V)   Standard _User;

	PointName(ID,Out T,Out S,V) Standard _User;
	 PointNameColor(ID,Out C,V)  Standard _User;
	PointNameFace(ID,Out F,V)   Standard _User;
	
     // These just to fool the linker
     Inactive PointLabel(0,Out 0,Out 0,0);
     Inactive PointLabelColor(0,Out 0,0);
     Inactive PointLabelFace(0,Out 0,0);
     Inactive SmallTextSize(0,Out 0,0);
     Inactive PointSize(0,Out 0,0);
	Inactive PointName(0,Out 0,Out 0,0);
	 Inactive PointNameColor(0,Out 0,0);
     Inactive PointNameFace(0,Out 0,0);

     // Graphical objects entailed by PointLabel
     SmallText(Out ID,Out X,Out Y,Out T,Out S,Out Center,V)
         For IP,YP: Point(IP,X,YP,V)
         If PointLabel(IP,T,S,V)
         Assign YP In {
             long PS;
             if (PointSize(IP,PS,V)) Y=YP+12+PS/2;
             else Y=YP+12;
             ID=IP-99999;
         };
     SmallTextColor(ID,Out C,V) For PointLabelColor(ID+99999,C,V);
     SmallTextFace(ID,Out F,V)  For PointLabelFace(ID+99999,F,V);

  // Graphical objects entailed by PointName
     SmallText(Out ID,Out X,Out Y,Out T,Out S,Out Center,V)
         For IP,YP: Point(IP,X,YP,V)
         If PointName(IP,T,S,V)
         Assign YP In {
             long PS;
             if (PointSize(IP,PS,V)) Y=YP+PS/3;
             else Y=YP;
             ID=IP-99999;
         };
     SmallTextColor(ID,Out C,V) For PointNameColor(ID+99999,C,V);
     SmallTextFace(ID,Out F,V)  For PointNameFace(ID+99999,F,V);


**/

/**  // Swarm visualization 

View(Out 1); // Current swarm position window (for level 1)
ViewOrigin(Out 1,Out 10, 1);

// Title
SmallText(Out 0,Out 50,Out 5,Out String, Out S,Out Left,1)
 Assign S="Current swarm"; 

// Grid
Rectangle(Out R,Out X,Out Y, Out 20,Out 20,1) // Grid
	For I:InRange(I,0,9)
	For J:InRange(J,0,9)
	Assign X=1+J*20 Y=10+I*20 R=10*I + J;
		
SmallText(Out 1,Out 180, Out 220, Out String, Out S, Out Left,1)
	Assign S="X1";
SmallText(Out 2,Out 0 , Out 5, Out String, Out S, Out Left,1)
	Assign S="X2";
SmallText(Out 1,Out 0, Out 220, Out String, Out S, Out Left,1)
	Assign S="0";

// Particles in the virtual space (X1,X2)
	Point(Out P,Out X1,Out X2,1) 
	For P:InRange(P,0,SWSIZE-1)
		Assign 	X1=1+frame.scale1*(PART[P].X1-frame.min1)
				X2=10+ SCALE - frame.scale2*(PART[P].X2 -frame.min2);

	PointSize(P,Out S,1) 
		Assign S = (PART[P].LABEL==LABEL_BEST) ? 20 :15 ;
			
	PointShape(P,Out Round,1) ;
	PointColor(P,Out C,1) Assign C=
	(PART[P].LABEL==LABEL_BEST) ? LightGreen  :
	(PART[P].F>frame.s[4]) ? Violet  :
	(PART[P].F>frame.s[3]) ? Blue :
	(PART[P].F>frame.s[2]) ? DarkMagenta :
	(PART[P].F>frame.s[1]) ? Red :
	(PART[P].F>frame.s[0]) ? Orange :
	(PART[P].F>frame.fmin) ? Yellow :
	LightGreen;
	
	PointName(P,Out Int,Out S,1) Assign S=PART[P].LABEL;
    PointNameColor(P,Out C,1) Assign C= (PART[P].F>frame.s[0]) ? White  : Black;

     //PointNameFace(P,Out Bold,1);
 
// Keep trace for the "winner"
	 Line (Out 0,Out X1,Out Y1 ,Out X2 ,Out Y2,1)
		For I:InRange(I,0,NB_BEST-2)
			Assign X1=  (NB_BEST>2) ? 1+frame.scale1*(L[I][0]-frame.min1) : 0
			Assign Y1= (NB_BEST>2) ? 10+SCALE-frame.scale2*(L[I][1]-frame.min2) : 0
			Assign X2=(NB_BEST>2) ? 1+frame.scale1*(L[I+1][0]-frame.min1) : X1
			Assign Y2=(NB_BEST>2) ? 10+SCALE-frame.scale2*(L[I+1][1]-frame.min2):Y1;
	
	LineColor (0,Out LightGreen,1);


// --------------------

View(Out 2); // Best swarm position window (for level 1)
ViewOrigin(Out 1,Out 10, 2);

SmallText(Out 0,Out 50,Out 5,Out String, Out S,Out Left,2)
 Assign S="Best swarm";

Rectangle(Out R,Out X,Out Y, Out 20,Out 20,2)
	For I:InRange(I,0,9)
	For J:InRange(J,0,9)
	Assign X=1+J*20 Y=10+I*20 R=I+J;
	
SmallText(Out 1,Out 180, Out 220, Out String, Out S, Out Left,2)
	Assign S="X1";
SmallText(Out 2,Out 0 , Out 5, Out String, Out S, Out Left,2)
	Assign S="X2";
SmallText(Out 1,Out 0, Out 220, Out String, Out S, Out Left,1)
	Assign S="0";

	Point(Out P,Out X1,Out X2,2) 
	For P:InRange(P,0,SWSIZE-1)
		Assign 	X1=1+frame.scale1*(PART[P].XP1-frame.min1) 
				X2=10+ SCALE  - frame.scale2*(PART[P].XP2-frame.min2);

	PointSize(P,Out S,2) 
		Assign S = (PART[P].LABEL==LABEL_BEST) ? 20 :15 ;
		
	PointShape(P,Out Round,2) ;
	PointColor(P,Out C,2) Assign C=
	(PART[P].LABEL==LABEL_BEST) ? LightGreen  :
	(PART[P].FP>frame.s[4]) ? Violet  :
	(PART[P].FP>frame.s[3]) ? Blue :
	(PART[P].FP>frame.s[2]) ? DarkMagenta :
	(PART[P].FP>frame.s[1]) ? Red :
	(PART[P].FP>frame.s[0]) ? Orange :
	(PART[P].FP>frame.fmin) ? Yellow :
	LightGreen;
	
	PointName(P,Out Int,Out S,2) Assign S=PART[P].LABEL;
    PointNameColor(P,Out C,2) Assign C= (PART[P].F>frame.s[0]) ? White  : Black;
	
	 Line (Out 0,Out X1,Out Y1 ,Out X2 ,Out Y2,2)
		For I:InRange(I,0,NB_BEST-2)
			Assign X1=(NB_BEST>2) ? 1+frame.scale1*(LP[I][0]-frame.min1):0
			Assign Y1=(NB_BEST>2) ? 10+SCALE-frame.scale2*(LP[I][1]-frame.min2):0
			Assign X2=(NB_BEST>2) ? 1+frame.scale1*(LP[I+1][0]-frame.min1) : 0
			Assign Y2=(NB_BEST>2) ? 10+SCALE-frame.scale2*(LP[I+1][1]-frame.min2):0;
	
	LineColor (0,Out LightGreen,2);
	
// --------------------

Graph(Out 3); // Swarm topology (for level 1)
Directed(3);
SmallText(Out 0,Out 50,Out 5,Out String, Out S,Out Left,3)
 Assign S="Swarm topology";

Node(Out N,3) For N:InRange(N,0,SWSIZE-1);

NodeColor(N,Out C ,3)
	Assign C=
	(PART[N].LABEL==LABEL_BEST) ? LightGreen  :
	(PART[N].F>frame.s[4]) ? Violet  :
	(PART[N].F>frame.s[3]) ? Blue :
	(PART[N].F>frame.s[2]) ? DarkMagenta :
	(PART[N].F>frame.s[1]) ? Red :
	(PART[N].F>frame.s[0]) ? Orange :
	(PART[N].F>frame.fmin) ? Yellow :
	LightGreen;
//NodeFrame(N,Out LightGreen,Out T,3)  Assign T=(PART[N].LABEL==LABEL_BEST) ? 5:0;
NodeName (N,Out Int,Out S,3) Assign S=PART[N].LABEL;


Arc(X,Y,3) If (PART[X].LABEL_H==PART[Y].LABEL);
//ArcOffset(X,Y,Out K,3) Assign K= (X==Y) ? 20 :  0 ;

//--------------
View(Out 4); // Coordinates of the "winner" (Star representation)
ViewOrigin(Out 1,Out 10, 4);
SmallText(Out 0,Out 10,Out 1,Out String, Out S,Out Left,4)
 Assign S="Winner's coordinates (star representation)";

// One line for each dimension
Line(Out 0,Out 100,Out 120,Out X2,Out Y2,4) 
	For I:InRange(I,0,DIM-1)
		Assign X2=100*(1+COS[I])
		Assign Y2=100*(1-SIN[I])+20 ;
		
// Axis numbers		
SmallText(Out 0,Out X,Out Y,Out Int,Out S,Out Left,4)
	For I:InRange(I,0,DIM-1)
		Assign S=I+1
		Assign X=100*(1+COS[I])
		Assign Y=100*(1-SIN[I]) +20 ;
	
// Scale on [0,1] for all coordinates
Point(Out I,Out X,Out Y,4) // 
	For I:InRange(I,0,DIM-1)
		Assign X=100*(1+COS[I]*(POS_BEST[I]-XMIN)/(XMAX-XMIN))
		Assign Y=100*(1-SIN[I]*(POS_BEST[I]-XMIN)/(XMAX-XMIN)) +20 ;

PointSize(P,Out S,4) 
	Assign S = 10 ;
		
PointShape(P,Out Round,4) ;
PointColor(P,Out C,4) 
	Assign C=LightGreen;
**/

// Subprograms
void			visu_2D(FILE *f_visu);
struct frame	visu_frame(FILE *f_visu);
struct graph	visu_graph(FILE *f_swarm);
void			visu_G(FILE *f_swarm);

// ----------------------------------------------------------------------------- MAIN
void main ()
{
char filename1[256]="";
char filename2[256]="";
FILE	*f_swarm;
FILE 	*f_visu;

/** Not Inference ; **/

 printf("Enter visu file name: ");
     #ifdef LEONARDO
         GetFileDialog(filename1);
     #else
	 	scanf("%s",filename1);
     #endif
f_visu=fopen(filename1,"r");

/*
 printf("Enter swarm file name: ");
     #ifdef LEONARDO
         GetFileDialog(filename2);
     #else
	 	scanf("%s",filename2);
     #endif

f_swarm=fopen(filename2,"r");
*/

frame=visu_frame(f_visu); // Analyze data and define the frame
fclose (f_visu);


//G=visu_graph(f_swarm); // Analyze date and define the canonical graph
//fclose(f_swarm);

/** Inference; **/

f_visu=fopen(filename1,"r");  // Visualization 
//f_swarm=fopen(filename2,"r");

visu_2D(f_visu);
//visu_G(f_swarm);

fclose (f_visu);



}

// ----------------------------------------------------------------------------- VISU_2D
void visu_2D(FILE *f_visu)
{
int 	d;
int		i;
int		LABEL,LABEL_H;
int		next=0;
float	pos[Max_DD];

float 	X1,X2,F,XP1,XP2,FP;	

printf("\n\n Visualization");

NB_BEST=0;
for (d=0;d<DIM;d++)
	{
		POS_BEST[d]=NA;
	}


next_swarm:
/** Not VisualUpdate; **/
fscanf(f_visu,"%i",&SWSIZE);
if (SWSIZE<0) goto end;

fscanf(f_visu,"%i",&DIM);
printf("\n swarm size: %i",SWSIZE);
printf("  dim: %i",DIM);


for (i=0;i<SWSIZE;i++)
{
	fscanf(f_visu,"%i %f %f %f %f %f %f %i",&LABEL, &X1,&X2,&F,&XP1,&XP2,&FP,&LABEL_H);
	for (d=0;d<DIM;d++)
		fscanf(f_visu," %f",&pos[d]);
		
	PART[i].LABEL=LABEL;
	PART[i].X1=X1;
	PART[i].X2=X2;
	PART[i].F=F;
	PART[i].XP1=XP1;
	PART[i].XP2=XP2;
	PART[i].FP=FP;
	PART[i].LABEL_H=LABEL_H;
	
	// Elements to display the (future) best particle (the "winner")
	if (LABEL==LABEL_BEST) 
	{
		L[NB_BEST][0]=X1; // Trajectory for current "position"
		L[NB_BEST][1]=X2;
		LP[NB_BEST][0]=XP1; // Trajectory for previous best "position"
		LP[NB_BEST][1]=XP2;
			
		NB_BEST=NB_BEST+1;
		
	//printf("\n");
		for (d=0;d<DIM;d++)
		{
			POS_BEST[d]=pos[d];
	//printf(" %f",pos[d]);
		}	
			
	}
	
}

/** VisualUpdate; **/


if (next==0)
{
	printf("\n 0 = next step, 1 = continue: ");
	scanf("%i", &next);
}

goto next_swarm;


end:
/** Not VisualUpdate; **/
printf("\n 0 to finish");
scanf("%i", &next);

return;
}

// ----------------------------------------------------------------------------- VISU_FRAME
struct frame visu_frame(FILE *f_visu)
{
/*
Analyze the data file to find good parameters to display  it

struct	frame		{float s[5];float scale1;float scale2;};
*/

int			d;
float		f_max=0; 
float		f_min; // (Note: We assume f = 0 at the objective point)
int			i;
int			LABEL,LABEL_H;
float		x;
float		x1_max;
float		x2_max; 
float		x1_min;
float		x2_min;
float 		X1,X2,F,XP1,XP2,FP;	
struct frame w;

printf("\n Analyzing data. Please wait");

x1_max=NA;
x2_max=NA; 
x1_min=-NA;
x2_min=-NA;
f_max=NA;
f_min=-NA;
XMAX=NA;
XMIN=-NA;

next_swarm:

fscanf(f_visu,"%i",&SWSIZE);
if (SWSIZE<0) goto end;

fscanf(f_visu," %i",&DIM);
printf("\n swarm size: %i",SWSIZE);
printf("  dim: %i",DIM);

for (i=0;i<SWSIZE;i++)
{
	fscanf(f_visu,"%i %f %f %f %f %f %f %i",&LABEL,&X1,&X2,&F,&XP1,&XP2,&FP,&LABEL_H);
	if (X1>x1_max) x1_max=X1;
	if (X2>x2_max) x2_max=X2;
	if (X1<x1_min) x1_min=X1;
	if (X2<x2_min) x2_min=X2;
	if (F>f_max) f_max=F;
	if (F<f_min) 
	{
		f_min=F;
		LABEL_BEST=LABEL;
	}
	
	for (d=0;d<DIM;d++)
	{
		fscanf(f_visu," %f",&x);
		if (x>XMAX) XMAX=x;
		if (x<XMIN) XMIN=x;
	}
}

goto next_swarm;

end:

w.min1=x1_min;
w.min2=x2_min;
w.max2=x2_max;
w.fmin=f_min;

w.scale1=SCALE/(x1_max-x1_min); // On [0 SCALE]
w.scale2=SCALE/(x2_max-x2_min);

f_max=f_max/6;

for (i=0;i<5;i++)
{
	w.s[i]=(i+1)*f_max;

}

printf("\n x1: [%f, %f], x2: [%f, %f], f: [0,%f]",x1_min,x1_max, x2_min, x2_max, f_max);
printf("\n XMIN %f, XMAX %f",XMIN,XMAX);

// --------- To display the winner's coordinates
for (d=0;d<DIM;d++)
{
	COS[d]= cos (2*PI*d/DIM);
	SIN[d]= sin (2*PI*d/DIM);
}

return w;

}

// ----------------------------------------------------------------------------- VISU_GRAPH
struct graph	visu_graph(FILE *f_swarm)
{
/*

Analyze swarms

*/
struct graph 	G={0};
int				bidon;

fscanf(f_swarm,"%i %i",&bidon,&G.size);

return G;
}

// ----------------------------------------------------------------------------- VISU_G
void visu_G(FILE *f_swarm)
{
/*

VIsualize graphs

*/
char	end;
int		DD;
int 	i;
int		LABEL;
float 	X1,X2,F,XP1,XP2,FP;	

next_swarm:
/** Not VisualUpdate; **/
fscanf(f_swarm,"%i %i",&SWSIZE,&DD);

if (SWSIZE<0) goto end;

//printf("\n swarm size: %i",SWSIZE);

for (i=0;i<SWSIZE;i++)
{
	fscanf(f_swarm,"%i %f %f %f %f %f %f",&LABEL, &X1,&X2,&F,&XP1,&XP2,&FP);
	PART[i].X1=X1;
	PART[i].X2=X2;
	PART[i].F=F;
	PART[i].XP1=XP1;
	PART[i].XP2=XP2;
	PART[i].FP=FP;
}
/** VisualUpdate; **/

//scanf("%c", &next);

goto next_swarm;


end:
printf("\n Press Return to finish");
scanf("%c", &end);


return;
}
